@@ -9,13 +9,21 @@ use Time::HiRes qw(sleep time);
 use Data::Dumper;
 use LWP::UserAgent;
 
+sub new {
+    my ($class, $port) = @_;
+    
+    $port ||= 10_249; # randomish port
+    
+    return $class->SUPER::new($port);
+}
+
 sub handle_request {
     my ( $self, $cgi ) = @_;
     my $params = $cgi->Vars;
 
     # If we are on port 8081 then we are a proxy - we should forward the
     # requests.
-    return act_as_proxy(@_) if $self->port == 8081;
+    return act_as_proxy(@_) if $self->{is_proxy};
 
     # We should act as a final destination server and so expect an absolute URL.
     my $request_uri = $ENV{REQUEST_URI};
@@ -28,7 +36,8 @@ sub handle_request {
     # trickle tests.
     $self->stdout_handle->autoflush(1);
 
-    # warn "START REQUEST - " . time;
+     # warn "START REQUEST - " . time;
+     # warn Dumper($params);
 
     # Do the right thing depending on what is asked of us.
     if ( exists $params->{redirect} ) {
@@ -49,15 +58,16 @@ sub handle_request {
         sleep( $params->{delay} );
         print $cgi->header( -nph => 1 );
         print "Delayed for '$params->{delay}'.\n";
-
     }
 
     elsif ( exists $params->{trickle} ) {
 
+        print $cgi->header( -nph => 1 );
+
         my $trickle_for = $params->{trickle};
         my $finish_at   = time + $trickle_for;
 
-        print $cgi->header( -nph => 1 );
+        local $| = 1;
 
         while ( time <= $finish_at ) {
             print time . " trickle $$\n";
